package;

import haxegon.*;

class SituationChoiceCard
{
	public var id:String;
	public var text:String;
	public var image:String;
	
	public function new()
	{
	}
}

class SituationChoice extends Situation
{
	static inline var animMaxFrame:Int = 22;
	static inline var animCardOffset:Int = 4;
	
	var choiceTitle:String;
	var cards:Array<SituationChoiceCard>;
	var hoverCard:Int;
	var choiceResult:String;
	
	var appearing:Bool;
	var animationFrame:Int;
	
	public function new(_title:String)
	{
		super();
		
		choiceTitle = _title;
		cards = [];
		choiceResult = "";
	
		hoverCard = -1;
		
		appearing = true;
		animationFrame = 0;
	}
	
	public function addCard(_id:String, _cardText:String, _image:String)
	{
		var card:SituationChoiceCard = new SituationChoiceCard();
		card.id = _id;
		card.text = _cardText;
		card.image = _image;
		cards.push(card);
	}

	public function getCardX(_card:Int) : Int
	{
		var xStart:Int = 73;
		var xInc:Int = 126;
		
		switch (cards.length)
		{
			case 1:
				xStart = 262;
			case 2:
				xStart = 199;
				xInc = 126;
			case 3:
				xStart = 136;
				xInc = 126;
			case 4:
				xStart = 73;
				xInc = 126;
		}
		
		return xStart + _card * xInc;
	}
	
	private function getMouseCard() : Int
	{
		for (i in 0 ... cards.length)
		{
			var cardX:Int = getCardX(i);
			var cardY:Int = 107;
			
			if (Mouse.x >= cardX && Mouse.x <= cardX + 116 && Mouse.y >= cardY && Mouse.y <= cardY + 157)
			{
				return i;
			}
		}
		
		return -1;
	}
	
	public override function dismiss()
	{
		hoverCard = -1;
		appearing = false;
	}
	
	public override function repeat()
	{
		// Special case: if we repeat the "free time" choice, respond to home style
		for (i in 0 ... cards.length)
		{
			if (cards[i].id == "RELAX")
			{
				switch (GameSession.get().shelter)
				{
					case 0: cards[i].image = "card_home_homeless";
					case 1: cards[i].image = "card_home_tent";
					case 2: cards[i].image = "card_home_shack";
					case 3: cards[i].image = "card_home_house";
				}
			}
		}
		
		hoverCard = -1;
		
		choiceResult = "";
		appearing = true;
		animationFrame = 0;
	}
	
	public override function update()
	{
		if (appearing)
		{
			hoverCard = -1;
			
			if (animationFrame < animMaxFrame + animCardOffset*(cards.length - 1))
			{
				++animationFrame;
			}
			else
			{
				hoverCard = getMouseCard();
				
				// where's the mouse?
				if (Mouse.leftclick() && hoverCard != -1)
				{
					choiceResult = cards[hoverCard].id;
					appearing = false;
				}
			}
		}
		else
		{
			if (animationFrame > 0)
			{
				--animationFrame;
			}
		}
	}
	
	public function getChoiceResult() : String
	{
		return choiceResult;
	}
	
	public override function isComplete()
	{
		return (!appearing && animationFrame <= 0);
	}
	
	public override function render()
	{
		//Gfx.fillbox(0, 0, Gfx.screenwidth, Gfx.screenheight, 0xe5f4ff, 0.3 * Math.min(animationFrame / animMaxFrame, 1.0));

		var titleAnimationFrame:Int = animationFrame;
		var titleY:Int = 66;
		if (titleAnimationFrame < animMaxFrame)
		{
			titleY -= Std.int(120 * Math.pow(1 - titleAnimationFrame / animMaxFrame, 3));
		}
		
		Gfx.drawimage(Gfx.screenwidthmid - 183, titleY, "choice_title");
		CustomText.render(Std.int(Gfx.screenwidthmid - 0.5 * CustomText.width(choiceTitle)), titleY + 4, choiceTitle, 0);
		
		for (i in 0 ... cards.length)
		{
			var cardX:Int = getCardX(i);
			var cardY:Int = 107;
			
			var cardAnimationFrame:Int = animationFrame;
			if (appearing)
			{
				// right hand cards start at -X
				cardAnimationFrame -= animCardOffset * i;
			}
			else
			{
				// left hand cards start at -X
				cardAnimationFrame -= animCardOffset * (cards.length-1-i);
			}
			
			if (cardAnimationFrame < 0) cardAnimationFrame = 0;
			if (cardAnimationFrame > animMaxFrame) cardAnimationFrame = animMaxFrame;
			
			cardY += Std.int(260 * Math.pow(1 - cardAnimationFrame / animMaxFrame, 3));
			
			if (hoverCard == i)
			{
				Gfx.drawimage(cardX, cardY, "choice_card_lit");
			}
			else
			{
				Gfx.drawimage(cardX, cardY, "choice_card");
			}

			Gfx.drawimage(cardX + 9, cardY + 9, cards[i].image);
			
			var textX:Int = cardX + 58;
			var textY:Int = cardY + 129;
			textX -= Std.int(0.5 * CustomText.width(cards[i].text));
			CustomText.render(textX, textY, cards[i].text, 1);
		}
	}
}