package;

import haxegon.*;

class SituationEventButton
{
	public var id:String;
	public var text:String;
	
	public function new()
	{
	}
}

class SituationEventEffect
{
	var stat:String;
	var change:Int;
	
	public function apply()
	{
		if (stat == "curseLift")
		{
			GameSession.get().cursed = false;
			return;
		}
		
		if (stat == "ui")
		{
			if (change > 0) {
				GameSession.get().ui = 1;
			} else {
				GameSession.get().ui = 0;
			}
			return;
		}
		
		Reflect.setField(GameSession.get(), stat.toLowerCase(), Reflect.field(GameSession.get(), stat.toLowerCase()) + change);
	}

	public function isPositive() : Bool
	{
		return change > 0;
	}
	
	public function getText() : String
	{
		if (stat == "ui_hide_shop")
		{
			return "";
		}
		
		if (stat == "curseLift")
		{
			return "The curse has lifted!";
		}
		
		if (stat == "ui")
		{
			return "";
		}
		
		if (stat == "money")
		{
			if (change > 0)
			{
				return "earned $" + change;
			}
			else
			{
				return "spent $" + (-change);
			}
		}
		else
		{
			if (change > 0)
			{
				return "" + stat + " +" + change;
			}
			else
			{
				return "" + stat + " " + change;
			}
		}
	}
	
	public function new(_stat:String, _change:Int)
	{
		stat = _stat;
		change = _change;
	}
}

class SituationEvent extends Situation
{
	static inline var animMaxFrame:Int = 20;

	var eventTitle:String;
	var eventDesc:String;
	var eventImage:String;
	var eventEffects:Array<SituationEventEffect>;
	var eventButtons:Array<SituationEventButton>;
	
	var hasAppliedEffects:Bool;
	
	var hoverButton:Int;
	var choiceResult:String;
	
	var appearing:Bool;
	var animationFrame:Int;
	
	public function new(_title:String, _desc:String, _image:String)
	{
		super();
		
		appearing = false;
		animationFrame = 0;
		
		eventTitle = _title;
		eventDesc = _desc;
		eventImage = _image;
		eventEffects = [];
		eventButtons = [];
		choiceResult = "";
	
		hoverButton = -1;
		
		appearing = true;
		animationFrame = 0;
	}
	
	public function addEffect(_stat:String, _change:Int)
	{
		if (_stat == "health")
		{
			if (GameSession.get().health + _change >= 5)
			{
				_change = 5 - GameSession.get().health;
				if (_change == 0)
				{
					return;
				}
			}
		}
		
		eventEffects.push(new SituationEventEffect(_stat, _change));
	}
	
	public function addButton(_id:String, _text:String)
	{
		var button:SituationEventButton = new SituationEventButton();
		button.id = _id;
		button.text = _text;
		eventButtons.push(button);
	}
	
	public override function update()
	{
		if (!hasAppliedEffects)
		{
			if (!appearing || (appearing && animationFrame >= 0.5*animMaxFrame))
			{
				for (ef in eventEffects)
				{
					ef.apply();
				}
				
				hasAppliedEffects = true;
			}
		}
		
		if (appearing)
		{
			hoverButton = -1;
			
			if (animationFrame < animMaxFrame)
			{
				++animationFrame;
			}
			else
			{
				//hoverCard = getMouseCard();
				var eventX:Int = Gfx.screenwidthmid - 183;
				var eventY:Int = 90;
				if (Mouse.x >= eventX && Mouse.x <= eventX + 366 && Mouse.y >= eventY && Mouse.y <= eventY + 161)
				{
					hoverButton = 0;
				}
				
				//// where's the mouse?
				if (Mouse.leftclick() && hoverButton != -1)
				{
					choiceResult = eventButtons[hoverButton].id;
					appearing = false;
				}
			}
		}
		else
		{
			if (animationFrame > 0)
			{
				--animationFrame;
			}
		}
	}	
	
	public function getChoiceResult() : String
	{
		return choiceResult;
	}

	public override function dismiss()
	{
		hoverButton = -1;
		choiceResult = "";
		appearing = false;
	}
	
	public override function repeat()
	{
		hoverButton = -1;
		
		choiceResult = "";
		appearing = true;
		animationFrame = 0;
	}
	
	public override function isComplete()
	{
		return (!appearing && animationFrame <= 0);
	}

	public override function render()
	{
		//Gfx.fillbox(0, 0, Gfx.screenwidth, Gfx.screenheight, 0xe5f4ff, 0.3 * Math.min(animationFrame / animMaxFrame, 1.0));
		
		var eventX:Int = Gfx.screenwidthmid - 183;
		var eventY:Int = 90;
		eventY += Std.int(320 * Math.pow(1 - animationFrame / animMaxFrame, 3));
		
		Gfx.drawimage(eventX, eventY, "event_bg");
		Gfx.drawimage(eventX + 9, eventY + 34, eventImage);
		
		CustomText.render(Std.int(Gfx.screenwidthmid - 0.5 * CustomText.width(eventTitle)), eventY + 6, eventTitle, 1);

		Text.wordwrap = 247;
		CustomText.render(eventX + 111, eventY + 33, eventDesc, 0);
		var effectY:Int = eventY + 33 + Std.int(Text.height(eventDesc));
		Text.wordwrap = 640;

		var positiveEffects:String = "";
		var negativeEffects:String = "";
		
		for (ef in eventEffects)
		{
			if (ef.isPositive())
			{
				if (positiveEffects == "")
				{
					positiveEffects = ef.getText();
					positiveEffects = positiveEffects.substr(0, 1).toUpperCase() + positiveEffects.substr(1);
				}
				else
				{
					positiveEffects = positiveEffects + ", " + ef.getText();
				}
			}
			else
			{
				if (negativeEffects == "")
				{
					negativeEffects = ef.getText();
					negativeEffects = negativeEffects.substr(0, 1).toUpperCase() + negativeEffects.substr(1);
				}
				else
				{
					negativeEffects = negativeEffects + ", " + ef.getText();
				}
			}
		}
		
		if (positiveEffects != "")
		{
			CustomText.render(eventX + 111, effectY, positiveEffects, 1);
			effectY += 18;
		}
		if (negativeEffects != "")
		{
			CustomText.render(eventX + 111, effectY, negativeEffects, 2);
		}

		for (i in 0 ... eventButtons.length)
		{
			var buttonX:Int = eventX + 282 - 80 * (eventButtons.length - 1 - i);
			var buttonY:Int = eventY + 127;
			
			if (hoverButton != -1)
			{
				Gfx.drawimage(buttonX, buttonY, "event_button_lit");
			}
			else
			{
				Gfx.drawimage(buttonX, buttonY, "event_button");
			}
			
			CustomText.render(Std.int(buttonX + 38 - 0.5 * CustomText.width(eventButtons[i].text)), buttonY + 2, eventButtons[i].text, 1);
		}
	}	
}